# PCHPC: GDB + Valgrind exercise

## Tutorial source codes:
### matrix_init_cn.c
```c-like=
#include <stdio.h>
#include <malloc.h>
#define DIM (16*1024)

int main( int argc, char** argv ) {
    size_t i, j;
    int* mat = (int*)malloc( DIM * DIM * sizeof( int ) );
    for( i = 0; i < DIM; ++i )
         for( j = 0; j < DIM; ++j )
              mat[j*DIM+i] = 42;
    puts("TEST DONE");
    return 0;
}
```

### matrix_init_rw.c

```c-like=
#include <stdio.h>
#include <malloc.h>
#define DIM (16*1024)

int main( int argc, char** argv ) {
    size_t i, j;
    int* mat = (int*)malloc( DIM * DIM * sizeof( int ) );
    for( i = 0; i < DIM; ++i )
       for( j = 0; j < DIM; ++j )
            mat[i*DIM+j] = 42;
    puts("TEST DONE");
    return 0;
}
```

# Solutions
## Tutorial 1: GDB
* Start gdb with the TUI option then use help to see options for different TUI layouts
```bash=
gdb --tui
help layout
```
* Change the default assembly syntax by adding the ".gdbinit" file to your $HOME directory
```bash=
$cat > ~/.gdbinit << 'EOF'
set disassembly-flavor intel
EOF
```
* To introduce a breakpoint into your code during execution you can do the following:
```bash=
$(gdb) file yourBinaryFile # load your executable
$(gdb) b 9 # è.g. introduce breakpoint at source code line #9
$(gdb) run  # run your executable
$(gdb) clear # remove the breakpoint
```
* Print contents of some general purpose registers (See Intel Architecture Software Developer's Manual for general purpose registers)
```bash=
$(gdb) print $rax # e.g print the system call register
$(gdb) print $rdi # or the file descriptor
```

## Tutorial 2: Analyse memory patterns using Valgrind

* Compile ```matrix_init_rw.c```and ```matrix_init_cn.c```with the debug option ```-g```
```bash=
$ gcc -o matrix_init_rw matrix_init_rw.c -g
$ gcc -o matrix_init_cn matrix_init_cn.c -g
```

* Run and time the two executables using shell built-in time utility
```bash=
$ time ./matrix_init_rw
$ time ./matrix_init_cn
```

* Compare cache misses in each program using Valgrind's ```Cachegrind``` tool
```bash=
$ module load valgrind # load Valgrind
$ valgrind --tool=cachegrind ./matrix_init_rw # takes more time to execute due to increased cache misses
$ valgrind --tool=cachegrind ./matrix_init_cn # reduced cache misses compared to 'matrix_init_rw'. Executes relatively faster.
```

* Use ```-cachegrind-out-file```option to write Valgrind output to a file
```bash=
$ valgrind --tool=cachegrind ./matrix_init_cn -cachegrind-out-file outfile_cn
$ valgrind --tool=cachegrind ./matrix_init_rw -cachegrind-out-file outfile_rw
```
    * Note: By default cachegrind.out.<pid> (where <pid> is the program's process ID) is written.
